import React, { FC } from 'react';
import { Text, TouchableOpacity, View } from 'react-native';
import { Image } from 'expo-image';
import { useNavigation } from '@react-navigation/native';
import * as FileSystem from 'expo-file-system';

import { styles } from './styles';

import { getOnlineStatus } from 'src/storage';
import { AvatarWithInitials } from 'src/components';

import { API_HOST } from '../../../../constants';
import { getFontSize } from '../../../../utils';
import { Colors } from '../../../../theme';

import TickIcon from '../../../../../assets/icons/tick.svg';
import UNIcon from '../../../../../assets/icons/un_icon.svg';
import NMIcon from '../../../../../assets/icons/nm_icon.svg';
import TBTIcon from '../../../../../assets/icons/tbt.svg';

import { NAVIGATION_PAGES } from '../../../../types';

type Props = {
  avatar: string;
  first_name: string;
  last_name: string;
  date_of_birth: number;
  homebase_flag: string;
  homebase2_flag: string;
  index: number;
  score: any[];
  active_score: number;
  tbt_score?: number;
  tbt_rank?: number;
  badge_tbt?: number;
  badge_1281: number;
  badge_un: number;
  auth: number;
  userId: number;
};

//TODO: Profile
export const Profile: FC<Props> = ({
  avatar,
  first_name,
  last_name,
  date_of_birth,
  homebase_flag,
  homebase2_flag,
  index,
  score,
  active_score,
  tbt_score,
  tbt_rank,
  badge_tbt,
  badge_1281,
  badge_un,
  auth,
  userId
}) => {
  const navigation = useNavigation();

  const scoreNames = ['NM', 'DARE', 'UN', 'UN+', 'TCC', 'DEEP', 'YES', 'SLOW', 'WHS', 'KYE', 'TBT'];
  const isOnline = getOnlineStatus();
  const avatarBaseUri = isOnline
    ? `${API_HOST}/img/avatars/`
    : `${FileSystem.documentDirectory}avatars/`;
  const flagBaseUri = isOnline
    ? `${API_HOST}/img/flags_new/`
    : `${FileSystem.documentDirectory}flags/`;

  const TBRanking = () => {
    const colors = [
      'rgba(237, 147, 52, 1)',
      'rgba(128, 128, 128, 1)',
      'rgba(211, 211, 211, 1)',
      'rgba(187, 95, 5, 1)',
      '#808080'
    ];

    const Rank = ({ color }: { color: string }) => (
      <View style={[styles.badge, { backgroundColor: color }]}>
        <TBTIcon />
      </View>
    );

    return tbt_rank && tbt_rank >= 1 ? (
      <View
        style={{
          display: 'flex',
          marginTop: 10
        }}
      >
        <View style={{ display: 'flex', alignItems: 'center' }}>
          {badge_tbt ? <Rank color={colors[tbt_rank - 1]} /> : null}
          <Text style={[styles.titleText, { flex: 0 }]}>TBT # {tbt_rank}</Text>
        </View>
      </View>
    ) : null;
  };

  const EmptyScore: FC<{ scoreName: string }> = ({ scoreName }) => {
    return (
      <View
        style={{
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          flexDirection: 'column',
          marginBottom: 10
        }}
      >
        <Text style={[styles.scoreHeaderText, { flex: 0 }]}>-</Text>
        <Text style={[styles.scoreNameText, { flex: 0 }]}>{scoreName}</Text>
      </View>
    );
  };

  return (
    <View>
      <TouchableOpacity
        onPress={() =>
          navigation.navigate(...([NAVIGATION_PAGES.PUBLIC_PROFILE_VIEW, { userId }] as never))
        }
      >
        <View style={{ alignItems: 'center', flexDirection: 'row', gap: 10, marginBottom: 5 }}>
          <View style={{ paddingLeft: 20 }}>
            <View style={{ marginRight: 5 }}>
              <Text style={styles.rankText}>{index + 1}</Text>
            </View>
            {avatar ? (
              <Image
                style={{ borderRadius: 24, width: 48, height: 48 }}
                source={{ uri: avatarBaseUri + avatar }}
              />
            ) : (
              <AvatarWithInitials
                text={`${first_name[0] ?? ''}${last_name[0] ?? ''}`}
                flag={flagBaseUri + homebase_flag}
                size={48}
              />
            )}
          </View>
          <View style={{ gap: 5, width: '75%' }}>
            <Text style={[styles.headerText, { fontSize: getFontSize(14), flex: 0 }]}>
              {first_name ?? ''} {last_name ?? ''}
            </Text>
            <View
              style={{ display: 'flex', justifyContent: 'space-between', flexDirection: 'row' }}
            >
              <View
                style={{
                  display: 'flex',
                  flexDirection: 'row',
                  gap: 10,
                  alignItems: 'center'
                }}
              >
                <Text
                  style={{ color: Colors.DARK_BLUE, fontWeight: '600', fontSize: getFontSize(12) }}
                >
                  Age: {date_of_birth ?? ''}
                </Text>
                <Image source={{ uri: flagBaseUri + homebase_flag }} style={styles.countryFlag} />
                {homebase2_flag && homebase2_flag !== homebase_flag ? (
                  <Image
                    source={{ uri: flagBaseUri + homebase2_flag }}
                    style={[styles.countryFlag, { marginLeft: -15 }]}
                  />
                ) : null}
                <View style={{ display: 'flex', flexDirection: 'row', gap: 3, marginLeft: 10 }}>
                  {auth ? <TickIcon /> : null}
                  {badge_un ? <UNIcon /> : null}
                  {badge_1281 ? <NMIcon /> : null}
                </View>
              </View>
            </View>
          </View>
        </View>
      </TouchableOpacity>
      <View style={styles.rankingWrapper}>
        <View style={styles.nmWrapper}>
          <Text style={{ fontSize: 20, color: Colors.DARK_BLUE, fontWeight: '700' }}>
            {score[active_score]}
          </Text>
          <Text style={{ fontSize: 14, color: Colors.DARK_BLUE, fontWeight: '500' }}>
            {scoreNames[active_score]}
          </Text>
        </View>

        <View style={styles.rankingScoresWrapper}>
          {score.map((number, index) => {
            if (scoreNames[index] === 'SLOW' && number >= 4500)
              return <EmptyScore scoreName={scoreNames[index]} />;
            if (scoreNames[index] === 'YES' && number >= 10000)
              return <EmptyScore scoreName={scoreNames[index]} />;
            if (!number) return <EmptyScore scoreName={scoreNames[index]} />;
            if (index === active_score) return;
            return (
              <View
                style={{
                  display: 'flex',
                  alignItems: 'center',
                  justifyContent: 'center',
                  flexDirection: 'column',
                  marginBottom: 10,
                  width: 40
                }}
              >
                <Text style={[styles.scoreHeaderText, { flex: 0 }]}>{number}</Text>
                <Text style={[styles.scoreNameText, { flex: 0 }]}>{scoreNames[index]}</Text>
              </View>
            );
          })}
        </View>

        <View style={{ marginRight: 15, marginLeft: 15 }}>
          <TBRanking />
        </View>
      </View>
    </View>
  );
};
